@@ -11,7 +11,6 @@ from __future__ import absolute_import
 
 from mercurial.i18n import _
 from mercurial import (
-    cmdutil,
     registrar,
 )
 
@@ -22,13 +21,110 @@ from . import (
 )
 
 cmdtable = {}
-command = cmdutil.command(cmdtable)
+command = registrar.command(cmdtable)
 # Note for extension authors: ONLY specify testedwith = 'ships-with-hg-core' for
 # extensions which SHIP WITH MERCURIAL. Non-mainline extensions should
 # be specifying the version(s) of Mercurial they are tested with, or
 # leave the attribute unspecified.
 testedwith = 'ships-with-hg-core'
 
+configtable = {}
+configitem = registrar.configitem(configtable)
+
+configitem('convert', 'cvsps.cache',
+    default=True,
+)
+configitem('convert', 'cvsps.fuzz',
+    default=60,
+)
+configitem('convert', 'cvsps.logencoding',
+    default=None,
+)
+configitem('convert', 'cvsps.mergefrom',
+    default=None,
+)
+configitem('convert', 'cvsps.mergeto',
+    default=None,
+)
+configitem('convert', 'git.committeractions',
+    default=lambda: ['messagedifferent'],
+)
+configitem('convert', 'git.extrakeys',
+    default=list,
+)
+configitem('convert', 'git.findcopiesharder',
+    default=False,
+)
+configitem('convert', 'git.remoteprefix',
+    default='remote',
+)
+configitem('convert', 'git.renamelimit',
+    default=400,
+)
+configitem('convert', 'git.saverev',
+    default=True,
+)
+configitem('convert', 'git.similarity',
+    default=50,
+)
+configitem('convert', 'git.skipsubmodules',
+    default=False,
+)
+configitem('convert', 'hg.clonebranches',
+    default=False,
+)
+configitem('convert', 'hg.ignoreerrors',
+    default=False,
+)
+configitem('convert', 'hg.revs',
+    default=None,
+)
+configitem('convert', 'hg.saverev',
+    default=False,
+)
+configitem('convert', 'hg.sourcename',
+    default=None,
+)
+configitem('convert', 'hg.startrev',
+    default=None,
+)
+configitem('convert', 'hg.tagsbranch',
+    default='default',
+)
+configitem('convert', 'hg.usebranchnames',
+    default=True,
+)
+configitem('convert', 'ignoreancestorcheck',
+    default=False,
+)
+configitem('convert', 'localtimezone',
+    default=False,
+)
+configitem('convert', 'p4.encoding',
+    default=lambda: convcmd.orig_encoding,
+)
+configitem('convert', 'p4.startrev',
+    default=0,
+)
+configitem('convert', 'skiptags',
+    default=False,
+)
+configitem('convert', 'svn.debugsvnlog',
+    default=True,
+)
+configitem('convert', 'svn.trunk',
+    default=None,
+)
+configitem('convert', 'svn.tags',
+    default=None,
+)
+configitem('convert', 'svn.branches',
+    default=None,
+)
+configitem('convert', 'svn.startrev',
+    default=0,
+)
+
 # Commands definition was moved elsewhere to ease demandload job.
 
 @command('convert',
@@ -182,8 +278,8 @@ def convert(ui, src, dest=None, revmapfile=None, **opts):
 
     where "original_branch_name" is the name of the branch in the
     source repository, and "new_branch_name" is the name of the branch
-    is the destination repository. No whitespace is allowed in the
-    branch names. This can be used to (for instance) move code in one
+    is the destination repository. No whitespace is allowed in the new
+    branch name. This can be used to (for instance) move code in one
     repository from "default" to a named branch.
 
     Mercurial Source
@@ -229,6 +325,12 @@ def convert(ui, src, dest=None, revmapfile=None, **opts):
         part of a changeset then the default may not be long enough.
         The default is 60.
 
+    :convert.cvsps.logencoding: Specify encoding name to be used for
+        transcoding CVS log messages. Multiple encoding names can be
+        specified as a list (see :hg:`help config.Syntax`), but only
+        the first acceptable encoding in the list is used per CVS log
+        entries. This transcoding is executed before cvslog hook below.
+
     :convert.cvsps.mergeto: Specify a regular expression to which
         commit log messages are matched. If a match occurs, then the
         conversion process will insert a dummy revision merging the
